#!/bin/sh

# Berkeley Open Infrastructure for Network Computing
# http://boinc.berkeley.edu
# Copyright (C) 2015 University of California
#
# This is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation;
# either version 2.1 of the License, or (at your option) any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details.
#
# To view the GNU Lesser General Public License visit
# http://www.gnu.org/copyleft/lesser.html
# or write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# Script to build Macintosh Universal Binary library of fftw-3.3.4 for
# use in building SETI_BOINC.
#
# by Charlie Fenton 5/2/06
# updated for OS 10.4 SDK and SETI@home v7 on 4/9/11
# updated for Xcode 6, fftw-3.3.4 and SETI@home v8 on 12/18/15
# Added AVX support and set optimization level to -O3 on 12/24/15
#
## This script requires OS 10.6 or later
#
## If you drag-install Xcode 4.3 or later, you must have opened Xcode 
## and clicked the Install button on the dialog which appears to 
## complete the Xcode installation before running this script.
#
## In Terminal, CD to the fftw-3.3.4 directory.
##     cd [path]/lib/fftw-3.3.4/
## then run this script:
##     source [path]/buildfftw-3.3.4 [ -clean ]
##
## the -clean argument will force a full rebuild.
#

if [ "$1" != "-clean" ]; then
  if [ -f .libs/libfftw3f_x86_64.a ] && [ -f .libs/libfftw3f_i386.a ] && [ -f .libs/libfftw3f.a ]; then
    
    echo "buildfftw-3.3.4 already built"
    return 0
  fi
fi

GCCPATH=`xcrun -find gcc`
if [  $? -ne 0 ]; then
    echo "ERROR: can't find gcc compiler"
    return 1
fi

GPPPATH=`xcrun -find g++`
if [  $? -ne 0 ]; then
    echo "ERROR: can't find g++ compiler"
    return 1
fi

MAKEPATH=`xcrun -find make`
if [  $? -ne 0 ]; then
    echo "ERROR: can't find make tool"
    return 1
fi

TOOLSPATH1=${MAKEPATH%/make}

ARPATH=`xcrun -find ar`
if [  $? -ne 0 ]; then
    echo "ERROR: can't find ar tool"
    return 1
fi

TOOLSPATH2=${ARPATH%/ar}

export PATH="${TOOLSPATH1}":"${TOOLSPATH2}":$PATH

SDKPATH=`xcodebuild -version -sdk macosx Path`

# Build for i386 architecture
export CC="${GCCPATH}";export CXX="${GPPPATH}"
export LDFLAGS="-Wl,-syslibroot,${SDKPATH},-arch,i386"
export CPPFLAGS="-isysroot ${SDKPATH} -arch i386 -O3 -DMAC_OS_X_VERSION_MAX_ALLOWED=1040 -DMAC_OS_X_VERSION_MIN_REQUIRED=1040"
export CFLAGS="-isysroot ${SDKPATH} -arch i386 -O3 -DMAC_OS_X_VERSION_MAX_ALLOWED=1040 -DMAC_OS_X_VERSION_MIN_REQUIRED=1040"
export SDKROOT="${SDKPATH}"
export MACOSX_DEPLOYMENT_TARGET=10.4

./configure -C --enable-avx --enable-sse2 --enable-sse -enable-float --enable-fma --disable-fortran --enable-single --enable-threads --with-combined-threads --host=i386 --build=i386
if [  $? -ne 0 ]; then exit 1; fi

make clean

rm -f .libs/libfftw3f.a
rm -f .libs/libfftw3f_x86_64.a
rm -f .libs/libfftw3f_i386.a

make -e
if [  $? -ne 0 ]; then exit 1; fi
mv -f .libs/libfftw3f.a ./libfftw3f_x86_64.a
    
# Build for x86_64 architecture
make clean
if [  $? -ne 0 ]; then exit 1; fi

export CC="${GCCPATH}";export CXX="${GPPPATH}"
export LDFLAGS="-Wl,-syslibroot,${SDKPATH},-arch,x86_64"
export CPPFLAGS="-isysroot ${SDKPATH} -arch x86_64 -O3 -DMAC_OS_X_VERSION_MAX_ALLOWED=1050 -DMAC_OS_X_VERSION_MIN_REQUIRED=1050"
export CFLAGS="-isysroot ${SDKPATH} -arch x86_64 -O3 -DMAC_OS_X_VERSION_MAX_ALLOWED=1050 -DMAC_OS_X_VERSION_MIN_REQUIRED=1050"
export SDKROOT="${SDKPATH}"
export MACOSX_DEPLOYMENT_TARGET=10.5

./configure --enable-avx --enable-sse2 --enable-sse -enable-float --enable-fma --disable-fortran --enable-single --enable-threads --with-combined-threads --host=x86_64 --build=x86_64
if [  $? -ne 0 ]; then exit 1; fi

make -e
if [  $? -ne 0 ]; then exit 1; fi
mv -f .libs/libfftw3f.a .libs/libfftw3f_i386.a
mv -f ./libfftw3f_x86_64.a .libs/libfftw3f_x86_64.a
lipo -create .libs/libfftw3f_i386.a .libs/libfftw3f_x86_64.a -output .libs/libfftw3f.a
if [  $? -ne 0 ]; then exit 1; fi

export CC="";export CXX=""
export LDFLAGS=""
export CPPFLAGS=""
export CFLAGS=""
export SDKROOT=""

return 0
